// Importing required modules from d3
import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
  symbolCross,
  symbolDiamond,
  symbolSquare,
  schemeCategory10,
} from 'd3';

// Exporting a function named scatterPlot
export const scatterPlot = () => {
  // Define variables for width, height, data, xValue, yValue, margin, and radius
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;

  // Define the main function my, which will be called on a selection
  const my = (selection) => {
    // Create x and y scales based on the data and provided xValue and yValue functions
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    // Map the data to marks, which include x, y, and species information for each data point
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      species: d.mpg,
    }));

    // Create scales for shape and color based on the species information
    var shape = scaleOrdinal(
      data.map((d) => d.species),
      symbols.map((s) => symbol().type(s)()),
    );

    var color = scaleOrdinal(
      data.map((d) => d.species),
      schemeCategory10,
    );

    // Select and bind data to circles, using the 'dots' class
    const circles = selection
      .selectAll('.dots')
      .data(marks)
      .join(
        // Enter selection: append a path for each data point
        (enter) =>
          enter.append('path').attr('class', 'dots'),
        // Update selection: update existing data points
        (update) => update,
        // Exit selection: remove data points that are no longer in the data
        (exit) => exit.remove(),
      )
      // Set the shape and color of each data point based on the species
      .attr('d', (d) => shape(d.species))
      .attr('fill', (d) => color(d.species))
      // Translate each data point to its x, y position
      .attr('transform', (d) => `translate(${d.x},${d.y})`);

    // Create and position the y-axis
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));

    // Create and position the x-axis
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x));
  };

  // Getter/setter functions for width, height, data, xValue, yValue, margin, and radius
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my; // Return the main function
};
